package com.github.polpetta.mezzotre.telegram.command;

import com.github.polpetta.mezzotre.i18n.TemplateContentGenerator;
import com.github.polpetta.mezzotre.orm.model.CallbackQueryContext;
import com.github.polpetta.mezzotre.orm.model.TgChat;
import com.github.polpetta.mezzotre.telegram.callbackquery.Field;
import com.github.polpetta.mezzotre.util.Clock;
import com.github.polpetta.mezzotre.util.UUIDGenerator;
import com.github.polpetta.types.json.CallbackQueryMetadata;
import com.github.polpetta.types.json.ChatContext;
import com.pengrad.telegrambot.model.Update;
import com.pengrad.telegrambot.model.request.InlineKeyboardButton;
import com.pengrad.telegrambot.model.request.InlineKeyboardMarkup;
import com.pengrad.telegrambot.model.request.ParseMode;
import com.pengrad.telegrambot.request.BaseRequest;
import com.pengrad.telegrambot.request.SendMessage;
import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.Executor;
import java.util.stream.Collectors;
import javax.inject.Inject;
import javax.inject.Named;
import org.apache.commons.lang3.tuple.Pair;

public class Help implements Processor {

  private static final String TRIGGERING_STAGING_NAME = "/help";

  private final TemplateContentGenerator templateContentGenerator;
  private final Executor threadPool;
  private final Clock clock;
  private final Map<String, Processor> tgCommandProcessors;
  private final Map<String, com.github.polpetta.mezzotre.telegram.callbackquery.Processor>
      eventProcessor;
  private final UUIDGenerator uuidGenerator;

  @Inject
  public Help(
      TemplateContentGenerator templateContentGenerator,
      @Named("eventThreadPool") Executor threadPool,
      Clock clock,
      @Named("commandProcessor") Map<String, Processor> tgCommandProcessors,
      @Named("eventProcessors")
          Map<String, com.github.polpetta.mezzotre.telegram.callbackquery.Processor> eventProcessor,
      UUIDGenerator uuidGenerator) {
    this.templateContentGenerator = templateContentGenerator;
    this.threadPool = threadPool;
    this.clock = clock;
    this.tgCommandProcessors = tgCommandProcessors;
    this.eventProcessor = eventProcessor;
    this.uuidGenerator = uuidGenerator;
  }

  @Override
  public Set<String> getTriggerKeywords() {
    return Set.of(TRIGGERING_STAGING_NAME);
  }

  @Override
  public CompletableFuture<Optional<BaseRequest<?, ?>>> process(TgChat chat, Update update) {
    return CompletableFuture.supplyAsync(
        () -> {
          final String message =
              templateContentGenerator.mergeTemplate(
                  velocityContext -> {
                    velocityContext.put(
                        "commands",
                        tgCommandProcessors.values().stream()
                            .distinct()
                            .map(
                                p ->
                                    Pair.of(
                                        p.getTriggerKeywords().stream()
                                            .sorted()
                                            .collect(Collectors.toList()),
                                        p.getLocaleDescriptionKeyword()))
                            .collect(Collectors.toList()));
                  },
                  chat.getLocale(),
                  "template/telegram/help.vm");

          final ChatContext chatContext = chat.getChatContext();
          chatContext.setStage(TRIGGERING_STAGING_NAME);
          chatContext.setStep(0);
          chatContext.setPreviousMessageUnixTimestampInSeconds(clock.now());
          chat.setChatContext(chatContext);
          chat.setHasHelpBeenShown(true);
          chat.save();

          final SendMessage sendMessage =
              new SendMessage(chat.getId(), message).parseMode(ParseMode.Markdown);

          final String callBackGroupId = uuidGenerator.generateAsString();
          final InlineKeyboardButton[] collect =
              eventProcessor.values().stream()
                  .filter(
                      com.github.polpetta.mezzotre.telegram.callbackquery.Processor
                          ::canBeDirectlyInvokedByTheUser)
                  .filter(e -> e.getPrettyPrintLocaleKeyName().isPresent())
                  .map(
                      eventProcessor -> {
                        final CallbackQueryContext callbackQueryContext =
                            new CallbackQueryContext(
                                uuidGenerator.generateAsString(),
                                callBackGroupId,
                                new CallbackQueryMetadata.CallbackQueryMetadataBuilder()
                                    .withEvent(eventProcessor.getEventName())
                                    .withTelegramChatId(chat.getId())
                                    .withAdditionalProperty(
                                        Field.ShowHelp.InvokedFromHelpMessage.getName(), true)
                                    .build());
                        callbackQueryContext.save();

                        return new InlineKeyboardButton(
                                templateContentGenerator.getString(
                                    chat.getLocale(),
                                    eventProcessor.getPrettyPrintLocaleKeyName().get()))
                            .callbackData(callbackQueryContext.getId());
                      })
                  .toArray(InlineKeyboardButton[]::new);

          sendMessage.replyMarkup(new InlineKeyboardMarkup(collect));

          return Optional.of(sendMessage);
        },
        threadPool);
  }
}
